@@ -1,9 +1,11 @@
-# RCS Info        : $Id$
+#! perl
+
+# RCS Info        : $Id: GenBase.pm,v 1.25 2008/03/02 15:21:16 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Sat Oct  8 16:40:43 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Wed Jul 12 16:17:29 2006
-# Update Count    : 139
+# Last Modified On: Sun Mar  2 16:19:09 2008
+# Update Count    : 149
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -15,6 +17,9 @@ package EB::Report::GenBase;
 
 use strict;
 use EB;
+
+our $VERSION = sprintf "%d.%03d", q$Revision: 1.25 $ =~ /(\d+)/g;
+
 use IO::File;
 use EB::Format;
 
@@ -51,6 +56,9 @@ sub backend {
 	$gen = $1;
     }
 
+    # Override by explicit --generate option(s).
+    $gen = $opts->{generate} if $opts->{generate};
+
     # Infer from filename extension.
     my $t;
     if ( !defined($gen) && ($t = $opts->{output}) && $t =~ /\.([^.]+)$/ ) {
@@ -62,8 +70,8 @@ sub backend {
     # Fallback to text.
     $gen ||= "text";
 
-    # Build class and package name.
-    my $class = (ref($self)||$self) . "::" . ucfirst($gen);
+    # Build class and package name. Last chance to override...
+    my $class = $opts->{backend} || (ref($self)||$self) . "::" . ucfirst($gen);
     my $pkg = $class;
     $pkg =~ s;::;/;g;;
     $pkg .= ".pm";
@@ -90,7 +98,8 @@ sub backend {
 	$be->{fh} = IO::File->new_from_fd(fileno(STDOUT), "w");
     }
     if ( $cfg->val(qw(locale unicode), 0) ) {
-	binmode($be->{fh}, ":utf8");
+	require Encode;
+	binmode($be->{fh}, ":encoding(utf8)");
     }
 
     # Handle pagesize.
@@ -157,7 +166,7 @@ sub backend {
     if ( $be->{per_begin} lt $opendate ) {
 	die("?".__x("Datum {per} valt vr het begin van de administratie {begin}",
 		    per   => datefmt_full($be->{per_begin}),
-		    begin => datfmt_full($opendate))."\n");
+		    begin => datefmt_full($opendate))."\n");
     }
     if ( $be->{per_end} lt $opendate ) {
 	die("?".__x("Datum {per} valt vr het begin van de administratie {begin}",
