@@ -1,20 +1,26 @@
-# Html.pm -- 
-# RCS Info        : $Id$
+#! perl
+
+# Html.pm -- HTML backend for Reporters.
+# RCS Info        : $Id: Html.pm,v 1.16 2008/03/10 17:41:45 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Thu Dec 29 15:46:47 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu Jan 18 16:11:17 2007
-# Update Count    : 57
+# Last Modified On: Mon Mar 10 18:26:18 2008
+# Update Count    : 67
 # Status          : Unknown, Use with caution!
 
 package main;
 
+our $dbh;
 our $cfg;
 
 package EB::Report::Reporter::Html;
 
 use strict;
 use warnings;
+
+our $VERSION = sprintf "%d.%03d", q$Revision: 1.16 $ =~ /(\d+)/g;
+
 use EB;
 use EB::Format qw(datefmt_full);
 
@@ -65,17 +71,32 @@ sub add {
 
     print {$self->{fh}} ("<tr", $style ? " class=\"r_$style\"" : (), ">\n");
 
+    my $colspan = 0;
     foreach my $col ( @{$self->{_fields}} ) {
+
+	if ( $colspan > 1 ) {
+	    $colspan--;
+	    next;
+	}
+
 	my $fname = $col->{name};
 	my $value = defined($data->{$fname}) ? $data->{$fname} : "";
+	my $class = "c_$fname";
 
 	# Examine style mods.
-	# No style mods for HTML.
-	# if ( $style ) {
-	#    if ( my $t = $self->_getstyle($style, $fname) ) {
-	#    }
-	# }
-	print {$self->{fh}} ("<td class=\"c_$fname\">",
+	if ( $style ) {
+	    if ( my $t = $self->_getstyle($style, $fname) ) {
+		if ( $t->{class} ) {
+		    $class = $t->{class};
+		}
+		if ( $t->{colspan} ) {
+		    $colspan = $t->{colspan};
+		}
+	    }
+	}
+	print {$self->{fh}} ("<td class=\"$class\"",
+			     $colspan > 1 ? " colspan=\"$colspan\"" : "",
+			     ">",
 			     $value eq "" ? "&nbsp;" : $html->($value),
 			     "</td>\n");
     }
@@ -122,17 +143,25 @@ sub header {
        "<p class=\"subtitle\">", $html->($self->{_title2}), "<br>\n", $html->($self->{_title3l}), "</p>\n",
        "<table class=\"main\">\n");
 
-    print {$self->{fh}} ("<tr class=\"head\">\n");
-    foreach ( @{$self->{_fields}} ) {
-	print {$self->{fh}} ("<th class=\"h_", $_->{name}, "\">",
-			     $html->($_->{title}), "</th>\n");
+    if ( grep { $_->{title} =~ /\S/ } @{$self->{_fields}} ) {
+	print {$self->{fh}} ("<tr class=\"head\">\n");
+	foreach ( @{$self->{_fields}} ) {
+	    print {$self->{fh}} ("<th class=\"h_", $_->{name}, "\">",
+				 $_->{title} ? $html->($_->{title}) : "&nbsp'",
+				 "</th>\n");
+	}
+	print {$self->{fh}} ("</tr>\n");
     }
-    print {$self->{fh}} ("</tr>\n");
 
 }
 
 ################ Internal methods ################
 
+sub html {
+    my $self = shift;
+    _html(@_);
+}
+
 sub _html {
     HTML::Entities::encode(shift);
 }
