@@ -1,289 +1,18 @@
-#!/usr/bin/perl
-
-my $RCS_Id = '$Id: Shell.pm,v 1.89 2006/12/27 12:40:09 jv Exp $ ';
-
-# Author          : Johan Vromans
-# Created On      : Thu Jul  7 15:53:48 2005
-# Last Modified By: Johan Vromans
-# Last Modified On: Wed Dec 27 13:35:28 2006
-# Update Count    : 850
-# Status          : Unknown, Use with caution!
-
-################ Common stuff ################
+#! perl
 
 package main;
 
-use strict;
-
-# Package or program libraries, if appropriate.
-# $LIBDIR = $ENV{'LIBDIR'} || '/usr/local/lib/sample';
-# use lib qw($LIBDIR);
-# require 'common.pl';
-
-# Package name.
-use EekBoek;
-
-################ Configuration ################
-
 our $cfg;
 our $dbh;
 
-sub shell {
-
-# This will set up the config at 'use' time.
-use EB::Config $EekBoek::PACKAGE;
-
-if ( @ARGV && ( $ARGV[0] eq '-P' || $ARGV[0] =~ /^--?printcfg$/ ) ) {
-    shift(@ARGV);
-    printconf();
-    exit;
-}
-
-################ Command line parameters ################
-
-use Getopt::Long 2.13;
-
-# Command line options.
-my $interactive = -t;
-my $command;
-my $echo;
-my $dataset;
-my $createdb;			# create database
-my $createsampledb;		# create demo database
-my $schema;			# initialise w/ schema
-my $confirm = 0;
-my $journal = 0;
-my $inexport;			# in/export
-my $inex_file;			# file voor in/export
-my $inex_dir;			# directory voor in/export
-my $verbose = 0;		# verbose processing
-my $bky;
-
-# Development options (not shown with -help).
-my $debug = 0;			# debugging
-my $trace = 0;			# trace (show process)
-my $test = 0;			# test mode.
-
-# Process command line options.
-app_options();
-
-# Post-processing.
-$trace |= ($debug || $test);
-
-################ Presets ################
-
-my $TMPDIR = $ENV{TMPDIR} || $ENV{TEMP} || '/usr/tmp';
-
-################ The Process ################
-
-use EB;
-#use base qw(EB::Shell);
-
-my $app = $EekBoek::PACKAGE;
-my $userdir = glob("~/.".lc($app));
-mkdir($userdir) unless -d $userdir;
-
-$echo = "eb> " if $echo;
-if ( $createsampledb ) {
-    $dataset = "sample" unless defined $dataset;
-}
-else {
-    $dataset ||= $cfg->val(qw(database name), undef);
-}
-
-unless ( $dataset ) {
-    die("?"._T("Geen dataset opgegeven.".
-	       " Specificeer een dataset in de configuratiefile,".
-	       " of geef een dataset".
-	       " naam mee op de command line met \"--dataset=...\".").
-	"\n");
-}
-
-$cfg->newval(qw(database name), $dataset);
-
-use EB::DB;
-our $dbh = EB::DB->new(trace => $trace);
-
-if ( defined $inexport ) {
-    if ( $inexport ) {
-	$command = 1;
-	$createdb = 1;
-	@ARGV = qw(import --noclean);
-	push(@ARGV, "--file", $inex_file) if defined $inex_file;
-	push(@ARGV, "--dir", $inex_dir) if defined $inex_dir;
-    }
-    else {
-	$command = 1;
-	@ARGV = qw(export);
-	push(@ARGV, "--file", $inex_file) if defined $inex_file;
-	push(@ARGV, "--dir", $inex_dir) if defined $inex_dir;
-    }
-}
-
-if ( $createsampledb ) {
-    $command = 1;
-    $createdb = 1;
-    my $file = findlib("schema/sampledb.ebz");
-    die("?".__x("Geen demo gegevens: {ebz}",
-	       ebz => "schema/sampledb.ebz")."\n") unless $file;
-    @ARGV = qw(import --noclean);
-    push(@ARGV, "--file", $file);
-}
-
-if ( $createdb ) {
-    $dbh->createdb($dataset);
-    warn("%".__x("Lege dataset {db} is aangemaakt", db => $dataset)."\n");
-}
-
-if ( $schema ) {
-    require EB::Tools::Schema;
-    $dbh->connectdb(1);
-    EB::Tools::Schema->create($schema);
-    $dbh->setup;
-}
-
-exit(0) if $command && !@ARGV;
-
-my $shell = EB::Shell->new
-  ({ HISTFILE	  => $userdir."/history",
-     command	  => $command,
-     interactive  => $interactive,
-     verbose	  => $verbose,
-     trace	  => $trace,
-     journal	  => $journal,
-     echo	  => $echo,
-     prompt	  => lc($app),
-     boekjaar	  => $bky,
-   });
-
-$| = 1;
-
-$shell->run;
-
-################ Subroutines ################
-
-sub printconf {
-    return unless @ARGV > 0;
-    my $sec = "general";
-    if ( !GetOptions(
-		     'section=s' => \$sec,
-		     '<>' => sub {
-			 my $conf = shift;
-			 my $sec = $sec;
-			 ($sec, $conf) = ($1, $2) if $conf =~ /^(.+?):(.+)/;
-			 my $val = $cfg->val($sec, $conf, undef);
-			 print STDOUT ($val) if defined $val;
-			 print STDOUT ("\n");
-		     }
-		    ) )
-    {
-	app_ident();
-	print STDERR __x(<<EndOfUsage, prog => $0);
-Gebruik: {prog} { --printcfg | -P } { [ --section=secname ] var ... } ...
-
-    Print de waarde van configuratie-variabelen.
-    Met --section=secname worden de eropvolgende variabelen
-    gezocht in sectie [secname].
-    Ook: secname:variabele.
-EndOfUsage
-    }
-}
-
-################ Subroutines ################
-
-sub app_options {
-    my $help = 0;		# handled locally
-    my $ident = 0;		# handled locally
-
-    # Process options, if any.
-    # Make sure defaults are set before returning!
-    return unless @ARGV > 0;
-
-    Getopt::Long::Configure(qw(no_ignore_case));
-
-    if ( !GetOptions(
-		     'command|c' => sub {
-			 $command = 1;
-			 die("!FINISH\n");
-		     },
-		     'import' => sub {
-			 $inexport = 1;
-		     },
-		     'export' => sub {
-			 $inexport = 0;
-		     },
-		     'createdb' => \$createdb,
-		     'createsampledb' => \$createsampledb,
-		     'define|D=s%' => sub {
-			 my ($opt, $key, $arg) = @_;
-			 if ( $key =~ /^(.+?)::?([^:]+)$/ ) {
-			     $cfg->newval($1, $2, $arg);
-			 }
-			 else {
-			     die(__x("Ongeldige aanduiding voor config setting: {arg}",
-				    arg => $key)."\n");
-			 }
-		     },
-		     'schema=s' => \$schema,
-		     'echo|e!'	=> \$echo,
-		     'ident'	=> \$ident,
-		     'journaal'	=> \$journal,
-		     'boekjaar=s'	=> \$bky,
-		     'verbose'	=> \$verbose,
-		     'db|dataset=s' => \$dataset,
-		     'dir=s'	=> \$inex_dir,
-		     'file=s'	=> \$inex_file,
-		     'trace'	=> \$trace,
-		     'help|?'	=> \$help,
-		     'debug'	=> \$debug,
-		    ) or $help )
-    {
-	app_usage(2);
-    }
-    app_usage(2) if @ARGV && !$command;
-    app_ident() if $ident;
-}
-
-sub app_ident {
-    return;
-    print STDERR (__x("Dit is {pkg} [{name} {version}]",
-		      pkg     => $EekBoek::PACKAGE,
-		      name    => "Shell",
-		      version => $EekBoek::VERSION) . "\n");
-}
-
-sub app_usage {
-    my ($exit) = @_;
-    app_ident();
-    print STDERR __x(<<EndOfUsage, prog => $0);
-Gebruik: {prog} [options] [file ...]
-
-    --command  -c       voer de rest van de opdrachtregel uit als command
-    --echo  -e          toon ingelezen opdrachten
-    --journaal          toon de journaalregels na elke opdracht
-    --dataset=DB        specificeer database
-    --db=DB             specificeer database
-    --boekjaar=XXX	specificeer boekjaar
-    --createdb		maak nieuwe database aan
-    --createsampledb	maak nieuwe demo database aan
-    --schema=XXX        initialisser database met schema
-    --import            importeer een nieuwe administratie
-    --export            exporteer een administratie
-    --dir=XXX           directory voor im/export
-    --file=XXX          bestand voor im/export
-    --define=XXX -D     definieer configuratiesetting
-    --help		deze hulpboodschap
-    --ident		toon identificatie
-    --verbose		geef meer uitgebreide information
-EndOfUsage
-    exit $exit if defined $exit && $exit != 0;
-}
-
-}
-
 package EB::Shell;
 
 use strict;
+use warnings;
+
+our $VERSION = sprintf "%d.%03d", q$Revision: 1.105 $ =~ /(\d+)/g;
+
+use EB;
 
 my $bky;			# current boekjaar (if set)
 
@@ -359,6 +88,7 @@ sub bky_msg {
     my $sth = $dbh->sql_exec("SELECT bky_code".
 			     " FROM Boekjaren".
 			     " WHERE bky_end < ?".
+			     " AND NOT bky_opened IS NULL".
 			     " AND bky_closed IS NULL".
 			     " ORDER BY bky_begin",
 			     defined $bky ?
@@ -479,12 +209,15 @@ sub _plug_cmds {
 	};
 	if ( $dbk_type == DBKTYPE_INKOOP ) {
 	    $dbk_v_pat .= lc($dbk_desc)."|";
+	    $dbk_pat .= lc($dbk_desc)."|";
 	}
 	elsif ( $dbk_type == DBKTYPE_VERKOOP ) {
 	    $dbk_i_pat .= lc($dbk_desc)."|";
+	    $dbk_pat .= lc($dbk_desc)."|";
 	}
 	else {
 	    $dbk_bkm_pat .= lc($dbk_desc)."|";
+	    $dbk_pat .= lc($dbk_desc)."|";
 	}
     }
 
@@ -513,8 +246,9 @@ sub _plug_cmds {
 	*{"help_btwaangifte"} = \&_help_btwaangifte;
     }
 
-    $dbk_pat = $dbk_i_pat.$dbk_v_pat.$dbk_bkm_pat;
-    chop foreach ($dbk_pat, $dbk_i_pat, $dbk_v_pat, $dbk_bkm_pat);
+    foreach ($dbk_pat, $dbk_i_pat, $dbk_v_pat, $dbk_bkm_pat) {
+	chop if $_;
+    }
 }
 
 sub _help {
@@ -592,27 +326,48 @@ sub _state {
 }
 
 sub do_trace {
-    my ($self, $state) = @_;
-    return unless argcnt(scalar(@_), 1, 2);
+    my ($self, @args) = @_;
+    my $opts = { quiet => 0 };
+    return unless
+    parse_args(\@args,
+	       [ 'quiet!' ],
+	       $opts);
+    return unless argcnt(@args, 0, 1);
+    my $state = shift(@args);
     $self->{trace} = _state($self->{trace}, $state);
 
     if ( $dbh ) {
 	$dbh->trace($self->{trace});
     }
+    return "" if $opts->{quiet};
     __x("SQL Trace: {state}", state => uc($self->{trace} ? _T("aan") : _T("uit")));
 }
 
 sub do_journal {
-    my ($self, $state) = @_;
-    return unless argcnt(scalar(@_), 1, 2);
+    my ($self, @args) = @_;
+    my $opts = { quiet => 0 };
+    return unless
+    parse_args(\@args,
+	       [ 'quiet!' ],
+	       $opts);
+    return unless argcnt(@args, 0, 1);
+    my $state = shift(@args);
     $self->{journal} = _state($self->{journal}, $state);
+    return "" if $opts->{quiet};
     __x("Journal: {state}", state => uc($self->{journal} ? _T("aan") : _T("uit")));
 }
 
 sub do_confirm {
-    my ($self, $state) = @_;
-    return unless argcnt(scalar(@_), 1, 2);
+    my ($self, @args) = @_;
+    my $opts = { quiet => 0 };
+    return unless
+    parse_args(\@args,
+	       [ 'quiet!' ],
+	       $opts);
+    return unless argcnt(@args, 0, 1);
+    my $state = shift(@args);
     $self->{confirm} = _state($self->{confirm}, $state);
+    return "" if $opts->{quiet};
     __x("Bevestiging: {state}", state => uc($self->{confirm} ? _T("aan") : _T("uit")));
 }
 
@@ -660,6 +415,7 @@ sub _add {
 		 d_boekjaar   => $bky || $dbh->adm("bky"),
 		 journal      => $self->{journal},
 		 totaal	      => undef,
+		 ref	      => undef,
 		 verbose      => $self->{verbose},
 		 confirm      => $self->{confirm},
 	       };
@@ -671,6 +427,7 @@ sub _add {
 		 'boekjaar=s',
 		 'journal!',
 		 'totaal=s',
+		 'ref=s',
 		 'confirm!',
 		 ( $dagboek_type == DBKTYPE_BANK
 		   || $dagboek_type == DBKTYPE_KAS ) ? ( 'saldo=s', 'beginsaldo=s' ) : (),
@@ -946,6 +703,7 @@ sub _do_btwaangifte {
 		 "definitief" => sub { $opts->{close} = 1 },
 		 EB::Report::GenBase->backend_options(EB::Report::BTWAangifte::, $opts),
 		 "noreport",
+		 "noround",
 	       ], $opts)
       or goto &_help_btwaangifte;
 
@@ -1007,6 +765,7 @@ sub do_debiteuren {
 	       [ "boekjaar=s",
 		 EB::Report::GenBase->backend_options(EB::Report::Debcrd::, $opts),
 		 'periode=s' => sub { periode_arg($opts, @_) },
+		 'openstaand',
 	       ], $opts);
 
     EB::Report::Debcrd->new->debiteuren(\@args, $opts);
@@ -1022,6 +781,7 @@ Opties:
 
   --periode <periode>         Periode
   --boekjaar=<code>           Selecteer boekjaar
+  --openstaand                Alleen met openstaande posten
 
 Zie verder "help rapporten" voor algemene informatie over aan te maken
 rapporten.
@@ -1041,6 +801,7 @@ sub do_crediteuren {
 	       [ "boekjaar=s",
 		 EB::Report::GenBase->backend_options(EB::Report::Debcrd::, $opts),
 		 'periode=s' => sub { periode_arg($opts, @_) },
+		 'openstaand',
 	       ], $opts);
 
     EB::Report::Debcrd->new->crediteuren(\@args, $opts);
@@ -1056,6 +817,7 @@ Opties:
 
   --periode=<periode>         Periode
   --boekjaar=<code>           Selecteer boekjaar
+  --openstaand                Alleen met openstaande posten
 
 Zie verder "help rapporten" voor algemene informatie over aan te maken
 rapporten.
@@ -1075,22 +837,26 @@ sub do_openstaand {
 	       [ "boekjaar=s",
 		 EB::Report::GenBase->backend_options(EB::Report::Open::, $opts),
 		 'per=s' => sub { date_arg($opts, @_) },
+		 'deb|debiteuren',
+		 'crd|crediteuren',
 	       ], $opts);
 
-    return unless argcnt(@args, 0);
-    EB::Report::Open->new->perform($opts);
+    return unless argcnt(@args, 0, 1);
+    EB::Report::Open->new->perform($opts, \@args);
 }
 
 sub help_openstaand {
     <<EOS;
 Toont een overzicht van openstaande posten.
 
-  openstaand [ <opties> ]
+  openstaand [ <opties> ] [ <relatie> ]
 
 Opties:
 
   --per=<datum>               Einddatum
   --boekjaar=<code>           Selecteer boekjaar
+  --deb --debiteuren          Alleen debiteuren
+  --crd --crediteuren         Alleen crediteuren
 
 Zie verder "help rapporten" voor algemene informatie over aan te maken
 rapporten.
@@ -1265,6 +1031,13 @@ sub do_import {
     }
 
     return unless argcnt(scalar(@args), 0);
+
+    if ( $opts->{clean} && !$dbh->feature("import") ) {
+	warn("?".__x("Database type {drv} ondersteunt niet het import commando. Gebruik de --import command line optie.",
+		     drv => $dbh->driverdb)."\n");
+	return;
+    }
+
     require EB::Import;
     EB::Import->do_import($self, $opts);
 
@@ -1344,40 +1117,8 @@ EOS
 sub do_dump_schema {
     my ($self, @args) = @_;
 
-    my $opts = { sql => 0,
-	       };
-
-    return unless
-    parse_args(\@args,
-	       [ 'sql!',
-	       ], $opts)
-      or goto &help_dump_schema;
-
-    require EB::Tools::Schema;
-
-    if ( $opts->{sql} ) {
-	return unless argcnt(scalar(@args), 1);
-	EB::Tools::Schema->dump_sql(@args);
-    }
-    else {
-	return unless argcnt(scalar(@args), 0);
-	EB::Tools::Schema->dump_schema;
-    }
-    "";
-}
-
-sub help_dump_schema {
     <<EOS;
-Reproduceert het schema van de huidige database en schrijft deze naar
-standaard uitvoer.
-
-  dump_schema                 Reproduceert het schema van de huidige database
-                              en schrijft deze naar standaard uitvoer.
-
-  dump_schema --sql <naam>    Creert losse SQL bestandjes om het genoemde
-                              schema aan te kunnen maken.
-
-Zie ook "help export".
+Deze opdracht is vervallen. Gebruik in plaats daarvan "export".
 EOS
 }
 
@@ -1408,7 +1149,7 @@ sub do_verwijder {
 	$cmd = EB::Booking::Decode->decode($id, { boekjaar => $opts->{boekjaar}, trail => 1, bsknr => 1, single => 1 });
     }
     my $res = EB::Booking::Delete->new->perform($id, $opts);
-    if ( $self->{interactive} && $res !~ /^[?!]/ ) {	# no error
+    if ( $res && $self->{interactive} && $res !~ /^[?!]/ ) {	# no error
 	$self->term->addhistory($cmd);
     }
     $res;
