@@ -1,12 +1,18 @@
-package EekBoek;
+#! perl
 
-$PACKAGE = 'EekBoek';
-$VERSION = '1.03.02';
+package EekBoek;
 
 # NOTE: This is a documentation-only module.
 
 use strict;
 
+# Minimal version to prevent many Unicode bugs.
+use 5.8.3;
+
+my $RCS_Id = '$Id: EekBoek.pm,v 1.63 2008/03/22 15:58:28 jv Exp $ ';
+our $PACKAGE = 'EekBoek';
+our $VERSION = '1.03.11';
+
 =head1 NAME
 
 EekBoek - Bookkeeping software for small and medium-size businesses
@@ -38,7 +44,7 @@ Web site: L<http://www.eekboek.nl>.
 
 =head1 COPYRIGHT AND DISCLAIMER
 
-This program is Copyright 2005-2007 by Squirrel Consultancy. All
+This program is Copyright 2005-2008 by Squirrel Consultancy. All
 rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -56,4 +62,4 @@ GNU General Public License or the Artistic License for more details.
 
 1;
 
-# $Id$
+# $Id: EekBoek.pm,v 1.63 2008/03/22 15:58:28 jv Exp $ 
